//======================================================================
//
// TabApi.c
//
// Elementare Kernel Driver per scovare e listare le native api
// di windows NT.
// La parte che fa il lavoro e' scritta in assembly per semplicita'
// Comunica con applicazioni Win32 attraverso le chiamate DeviceIo
// Il codice non e' ottimizzato per far capire esattamente le operazioni
// da compiere
// 
//
//   Autore    : Devil   (SPP Member)
//   Versione  : 1.00 
//
//======================================================================



#include "ntddk.h"
#include "stdarg.h"
#include "stdio.h"
#include <devioctl.h>
#include "kernel.h"
#include "tabapi.h"


// Macro attiva solo con la checked build 
#if DBG
#define DbgPrint(arg) DbgPrint arg
#else
#define DbgPrint(arg)
#endif

// Variabili Gloabli

InfoTabella gTabella;



NTSTATUS TabApiDispatchGeneral(IN PDEVICE_OBJECT DeviceObject,IN PIRP Irp );
VOID TabApiUnload(IN PDRIVER_OBJECT DriverObject);



//********************************************************************
//
// DriverEntry
//
// Punto di ingresso del Device Driver 
// 
// 
//
//**********************************************************************

NTSTATUS DriverEntry(IN PDRIVER_OBJECT  DriverObject,IN PUNICODE_STRING RegistryPath )
{
    PDEVICE_OBJECT         deviceObject        = NULL;
    NTSTATUS               ntStatus;
    WCHAR                  deviceNameBuffer[]  = L"\\Device\\TabApi";
    UNICODE_STRING         deviceNameUnicodeString;
    WCHAR                  deviceLinkBuffer[]  = L"\\DosDevices\\TabApi";
    UNICODE_STRING         deviceLinkUnicodeString;

    DbgPrint (("TabApi.SYS: Ingresso DriverEntry\n"));

    //
    // Creazione device
    //
	    
    RtlInitUnicodeString (&deviceNameUnicodeString,
			  deviceNameBuffer );
    
    ntStatus = IoCreateDevice (DriverObject,
			       0,
			       &deviceNameUnicodeString,
			       FILE_DEVICE_TABAPI,
			       0,
			       TRUE,
			       &deviceObject );

    if (NT_SUCCESS(ntStatus)) {

	 //
	 // Crea un Link Simbolico anche per le chiamate da programmi
	 // Win32/Dos
	 //

	 RtlInitUnicodeString (&deviceLinkUnicodeString,
			       deviceLinkBuffer );
	 ntStatus = IoCreateSymbolicLink (&deviceLinkUnicodeString,
					  &deviceNameUnicodeString );
	 if (!NT_SUCCESS(ntStatus))
	      DbgPrint (("TabApi.SYS: IoCreateSymbolicLink fallito\n"));

	 //
	 // Inizializzazione della tabella di dispatch che l'IO Manager
	 // utilizzera' per il driver
	 //

	 
	 DriverObject->MajorFunction[IRP_MJ_CREATE]         =
	 DriverObject->MajorFunction[IRP_MJ_CLOSE]          =
	  DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] 
			   = TabApiDispatchGeneral;
	 DriverObject->DriverUnload                         = TabApiUnload; 

	 return STATUS_SUCCESS;
    }
    else
       return ntStatus;






   
}




//**********************************************************************
//
// TabApiDispatchGeneral
//
// Qui mettiamo le risposte alle richieste di IOCTRL 
//
//
//**********************************************************************
NTSTATUS TabApiDispatchGeneral(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp )
{
    KIRQL OldIrql;
    ULONG ioControlCode;
    ULONG inputLength,outputLength;
    PVOID inputBuffer,outputBuffer;
    PVOID pPuntaMem;
    NTSTATUS Status=STATUS_SUCCESS;
    ULONG NumBytes;

    PIO_STACK_LOCATION CurrentIrpStack = IoGetCurrentIrpStackLocation(Irp);
    

    inputBuffer             = Irp->AssociatedIrp.SystemBuffer;
    inputLength       = CurrentIrpStack->Parameters.DeviceIoControl.InputBufferLength;
    outputBuffer            = Irp->AssociatedIrp.SystemBuffer;
    outputLength      = CurrentIrpStack->Parameters.DeviceIoControl.OutputBufferLength;
    ioControlCode           = CurrentIrpStack->Parameters.DeviceIoControl.IoControlCode;


       
    Irp->IoStatus.Information = 0;

    switch(CurrentIrpStack->MajorFunction)
	{
		case IRP_MJ_DEVICE_CONTROL:
			DbgPrint (("TabApi: IRP_MJ_DEVICE_CONTROL\n"));
			switch(ioControlCode)
			{
				case IOCTL_TABAPI_READBUFFER:
					
					memcpy(&gTabella,inputBuffer,inputLength);
					gTabella.NumServizi=KeServiceDescriptorTable.NumeroServizi;
					if (gTabella.Servizio<gTabella.NumServizi)
					{
						gTabella.IndirizzoTabellaServizi=(unsigned int)
						(KeServiceDescriptorTable.pTabellaServizi);
						gTabella.IndirizzoTabellaParametri=(unsigned int)
						(KeServiceDescriptorTable.pTabellaParametri);
						
						_asm mov gTabella.Segmento,CS
						_asm mov gTabella.SegmentoTabella,DS
						_asm push ecx
						_asm push eax
						_asm mov ecx,gTabella.Servizio
						_asm shl ecx,2
						_asm add ecx,gTabella.IndirizzoTabellaServizi
						_asm mov eax,[ecx]
						_asm mov gTabella.Offset,eax
						_asm mov ecx,gTabella.Servizio
						_asm add ecx,gTabella.IndirizzoTabellaParametri
						_asm xor eax,eax
						_asm mov al,[ecx]
						_asm mov gTabella.Parametro,eax
						_asm pop eax
						_asm pop ecx

						if (outputLength>=sizeof(InfoTabella))
						{
						   memcpy(outputBuffer,&gTabella,sizeof(InfoTabella));
						
						   Irp->IoStatus.Information =sizeof(InfoTabella);
						   Status=STATUS_SUCCESS;
						}
						else
						Status=STATUS_INVALID_BUFFER_SIZE;
					}
					else
					Status=STATUS_INVALID_PARAMETER;
					
				break;
				
						  
				default:
					Status=STATUS_INVALID_DEVICE_REQUEST;

				break;

			}

		break;
	}
	
	Irp->IoStatus.Status=Status;
    IoCompleteRequest( Irp, IO_NO_INCREMENT );
    return Status;

   
}

//*****************************************************

// Funzione chiamata all'atto di stoppare il device driver

//*****************************************************


VOID TabApiUnload(IN PDRIVER_OBJECT DriverObject)
{
    WCHAR                  deviceLinkBuffer[]  = L"\\DosDevices\\TabApi";
    UNICODE_STRING         deviceLinkUnicodeString;
   


    RtlInitUnicodeString (&deviceLinkUnicodeString,
			  deviceLinkBuffer
			  );

    IoDeleteSymbolicLink (&deviceLinkUnicodeString);
    IoDeleteDevice (DriverObject->DeviceObject);

   
}

